@@ -1,11 +1,13 @@
 <?php
 
-function get_bookmark($bookmark_id, $output = OBJECT) {
+function get_bookmark($bookmark_id, $output = OBJECT, $filter = 'raw') {
 	global $wpdb;
 
 	$bookmark_id = (int) $bookmark_id;
-	$link = $wpdb->get_row("SELECT * FROM $wpdb->links WHERE link_id = '$bookmark_id'");
-	$link->link_category = wp_get_link_cats($bookmark_id);
+	$link = $wpdb->get_row("SELECT * FROM $wpdb->links WHERE link_id = '$bookmark_id' LIMIT 1");
+	$link->link_category = array_unique( wp_get_object_terms($link_id, 'link_category', 'fields=ids') );
+
+	$link = sanitize_bookmark($link, $filter);
 
 	if ( $output == OBJECT ) {
 		return $link;
@@ -18,6 +20,22 @@ function get_bookmark($bookmark_id, $output = OBJECT) {
 	}
 }
 
+function get_bookmark_field( $field, $bookmark, $context = 'display' ) {
+	$bookmark = (int) $bookmark;
+	$bookmark = get_bookmark( $bookmark );
+
+	if ( is_wp_error($bookmark) )
+		return $bookmark;
+
+	if ( !is_object($bookmark) )
+		return '';
+
+	if ( !isset($bookmark->$field) )
+		return '';
+
+	return sanitize_bookmark_field($field, $bookmark->$field, $bookmark->link_id, $context);
+}
+
 // Deprecate
 function get_link($bookmark_id, $output = OBJECT) {
 	return get_bookmark($bookmark_id, $output);
@@ -26,15 +44,16 @@ function get_link($bookmark_id, $output = OBJECT) {
 function get_bookmarks($args = '') {
 	global $wpdb;
 
-	if ( is_array($args) )
-		$r = &$args;
-	else
-		parse_str($args, $r);
+	$defaults = array(
+		'orderby' => 'name', 'order' => 'ASC',
+		'limit' => -1, 'category' => '',
+		'category_name' => '', 'hide_invisible' => 1,
+		'show_updated' => 0, 'include' => '',
+		'exclude' => ''
+	);
 
-	$defaults = array('orderby' => 'name', 'order' => 'ASC', 'limit' => -1, 'category' => '',
-		'category_name' => '', 'hide_invisible' => 1, 'show_updated' => 0, 'include' => '', 'exclude' => '');
-	$r = array_merge($defaults, $r);
-	extract($r);
+	$r = wp_parse_args( $args, $defaults );
+	extract( $r, EXTR_SKIP );
 
 	$key = md5( serialize( $r ) );
 	if ( $cache = wp_cache_get( 'get_bookmarks', 'bookmark' ) )
@@ -75,8 +94,8 @@ function get_bookmarks($args = '') {
 		$exclusions .= ')';
 
 	if ( ! empty($category_name) ) {
-		if ( $cat_id = $wpdb->get_var("SELECT cat_ID FROM $wpdb->categories WHERE cat_name='$category_name' LIMIT 1") )
-			$category = $cat_id;
+		if ( $category = get_term_by('name', $category_name, 'link_category') )
+			$category = $category->term_id;
 	}
 
 	$category_query = '';
@@ -86,15 +105,15 @@ function get_bookmarks($args = '') {
 		if ( count($incategories) ) {
 			foreach ( $incategories as $incat ) {
 				if (empty($category_query))
-					$category_query = ' AND ( category_id = ' . intval($incat) . ' ';
+					$category_query = ' AND ( tt.term_id = ' . intval($incat) . ' ';
 				else
-					$category_query .= ' OR category_id = ' . intval($incat) . ' ';
+					$category_query .= ' OR tt.term_id = ' . intval($incat) . ' ';
 			}
 		}
 	}
 	if (!empty($category_query)) {
-		$category_query .= ')';
-		$join = " LEFT JOIN $wpdb->link2cat ON ($wpdb->links.link_id = $wpdb->link2cat.link_id) ";
+		$category_query .= ") AND taxonomy = 'link_category'";
+		$join = " LEFT JOIN $wpdb->term_relationships AS tr ON ($wpdb->links.link_id = tr.object_id) LEFT JOIN $wpdb->term_taxonomy as tt ON tt.term_taxonomy_id = tr.term_taxonomy_id";
 	}
 
 	if (get_option('links_recently_updated_time')) {
@@ -136,11 +155,72 @@ function get_bookmarks($args = '') {
 	$results = $wpdb->get_results($query);
 
 	$cache[ $key ] = $results;
-	wp_cache_add( 'get_bookmarks', $cache, 'bookmark' );
+	wp_cache_set( 'get_bookmarks', $cache, 'bookmark' );
 
 	return apply_filters('get_bookmarks', $results, $r);
 }
 
+function sanitize_bookmark($bookmark, $context = 'display') {
+	$fields = array('link_id', 'link_url', 'link_name', 'link_image', 'link_target', 'link_category',
+		'link_description', 'link_visible', 'link_owner', 'link_rating', 'link_updated',
+		'link_rel', 'link_notes', 'link_rss', );
+
+	$do_object = false;
+	if ( is_object($bookmark) )
+		$do_object = true;
+
+	foreach ( $fields as $field ) {
+		if ( $do_object )
+			$bookmark->$field = sanitize_bookmark_field($field, $bookmark->$field, $bookmark->link_id, $context);
+		else
+			$bookmark[$field] = sanitize_bookmark_field($field, $bookmark[$field], $bookmark['link_id'], $context);
+	}
+
+	return $bookmark;
+}
+
+function sanitize_bookmark_field($field, $value, $bookmark_id, $context) {
+	$int_fields = array('link_id', 'link_rating');
+	if ( in_array($field, $int_fields) )
+		$value = (int) $value;
+
+	$yesno = array('link_visible');
+	if ( in_array($field, $yesno) )
+		$value = preg_replace('/[^YNyn]/', '', $value);
+
+	if ( 'link_target' == $field ) {
+		$targets = array('_top', '_blank');
+		if ( ! in_array($value, $targets) )
+			$value = '';
+	}
+
+	if ( 'raw' == $context )
+		return $value;
+
+	if ( 'edit' == $context ) {
+		$format_to_edit = array('link_notes');
+		$value = apply_filters("edit_$field", $value, $bookmark_id);
+
+		if ( in_array($field, $format_to_edit) ) {
+			$value = format_to_edit($value);
+		} else {
+			$value = attribute_escape($value);
+		}
+	} else if ( 'db' == $context ) {
+		$value = apply_filters("pre_$field", $value);
+	} else {
+		// Use display filters by default.
+		$value = apply_filters($field, $value, $bookmark_id, $context);
+	}
+
+	if ( 'attribute' == $context )
+		$value = attribute_escape($value);
+	else if ( 'js' == $context )
+		$value = js_escape($value);
+
+	return $value;
+}
+
 function delete_get_bookmark_cache() {
 	wp_cache_delete( 'get_bookmarks', 'bookmark' );
 }
@@ -148,4 +228,4 @@ add_action( 'add_link', 'delete_get_bookmark_cache' );
 add_action( 'edit_link', 'delete_get_bookmark_cache' );
 add_action( 'delete_link', 'delete_get_bookmark_cache' );
 
-?>
\ No newline at end of file
+?>
