@@ -24,10 +24,10 @@ for ($i=1; $i <= $count; $i++) :
 
 	$content = '';
 	$content_type = '';
+	$content_transfer_encoding = '';
 	$boundary = '';
 	$bodysignal = 0;
-	$dmonths = array('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
-					 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
+	$dmonths = array('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
 	foreach ($message as $line) :
 		if (strlen($line) < 3) $bodysignal = 1;
 
@@ -40,6 +40,12 @@ for ($i=1; $i <= $count; $i++) :
 				$content_type = explode(';', $content_type);
 				$content_type = $content_type[0];
 			}
+			if (preg_match('/Content-Transfer-Encoding: /i', $line)) {
+				$content_transfer_encoding = trim($line);
+				$content_transfer_encoding = substr($content_transfer_encoding, 27, strlen($content_transfer_encoding)-14);
+				$content_transfer_encoding = explode(';', $content_transfer_encoding);
+				$content_transfer_encoding = $content_transfer_encoding[0];
+			}
 			if (($content_type == 'multipart/alternative') && (preg_match('/boundary="/', $line)) && ($boundary == '')) {
 				$boundary = trim($line);
 				$boundary = explode('"', $boundary);
@@ -56,19 +62,19 @@ for ($i=1; $i <= $count; $i++) :
 
 			// Set the author using the email address (To or Reply-To, the last used)
 			// otherwise use the site admin
-			if (preg_match('/From: /', $line) | preg_match('Reply-To: /', $line))  {
+			if (preg_match('/From: /', $line) | preg_match('/Reply-To: /', $line))  {
 				$author=trim($line);
-			if ( ereg("([a-zA-Z0-9\_\-\.]+@[\a-zA-z0-9\_\-\.]+)", $author , $regs) ) {
-				$author = $regs[1];
-				echo "Author = {$author} <p>";
-				$author = $wpdb->escape($author);
-				$result = $wpdb->get_row("SELECT ID FROM $wpdb->users WHERE user_email='$author' LIMIT 1");
-				if (!$result)
+				if ( ereg("([a-zA-Z0-9\_\-\.]+@[\a-zA-z0-9\_\-\.]+)", $author , $regs) ) {
+					$author = $regs[1];
+					echo "Author = {$author} <p>";
+					$author = $wpdb->escape($author);
+					$result = $wpdb->get_row("SELECT ID FROM $wpdb->users WHERE user_email='$author' LIMIT 1");
+					if (!$result)
+						$post_author = 1;
+					else
+						$post_author = $result->ID;
+				} else
 					$post_author = 1;
-				else
-					$post_author = $result->ID;
-			} else
-				$post_author = 1;
 			}
 
 			if (preg_match('/Date: /i', $line)) { // of the form '20 Mar 2002 20:32:37'
@@ -102,7 +108,7 @@ for ($i=1; $i <= $count; $i++) :
 		}
 	endforeach;
 
-	$subject = trim(str_replace(get_option('subjectprefix'), '', $subject));
+	$subject = trim($subject);
 
 	if ($content_type == 'multipart/alternative') {
 		$content = explode('--'.$boundary, $content);
@@ -111,11 +117,16 @@ for ($i=1; $i <= $count; $i++) :
 		$content = strip_tags($content[1], '<img><p><br><i><b><u><em><strong><strike><font><span><div>');
 	}
 	$content = trim($content);
+
+	if (stripos($content_transfer_encoding, "quoted-printable") !== false) {
+		$content = quoted_printable_decode($content);
+	}
+
 	// Captures any text in the body after $phone_delim as the body
 	$content = explode($phone_delim, $content);
 	$content[1] ? $content = $content[1] : $content = $content[0];
 
-	echo "<p><b>Content-type:</b> $content_type, <b>boundary:</b> $boundary</p>\n";
+	echo "<p><b>Content-type:</b> $content_type, <b>Content-Transfer-Encoding:</b> $content_transfer_encoding, <b>boundary:</b> $boundary</p>\n";
 	echo "<p><b>Raw content:</b><br /><pre>".$content.'</pre></p>';
 
 	$content = trim($content);
@@ -137,6 +148,8 @@ for ($i=1; $i <= $count; $i++) :
 	$post_data = add_magic_quotes($post_data);
 
 	$post_ID = wp_insert_post($post_data);
+	if ( is_wp_error( $post_ID ) ) 
+		echo "\n" . $post_ID->get_error_message();
 
 	if (!$post_ID) {
 		// we couldn't post, for whatever reason. better move forward to the next email
@@ -161,4 +174,4 @@ endfor;
 
 $pop3->quit();
 
-?>
\ No newline at end of file
+?>
